@@ -576,6 +576,8 @@ ALLEGRO_AUDIO_DRIVER _al_kcm_aqueue_driver = {
    _aqueue_set_voice_position,
 
    _aqueue_allocate_recorder,
-   _aqueue_deallocate_recorder
+   _aqueue_deallocate_recorder,
+
+   NULL
 };
 
