@@ -54,6 +54,8 @@ typedef struct PULSEAUDIO_VOICE
    char *buffer_end;
 } PULSEAUDIO_VOICE;
 
+static _AL_LIST* output_device_list;
+
 #define DEFAULT_BUFFER_SIZE   1024
 #define MIN_BUFFER_SIZE       128
 
@@ -73,16 +75,41 @@ static unsigned int get_buffer_size(const ALLEGRO_CONFIG *config)
    return DEFAULT_BUFFER_SIZE;
 }
 
+static void _output_device_list_dtor(void* value, void* userdata)
+{
+   (void)userdata;
+
+   ALLEGRO_AUDIO_DEVICE* device = (ALLEGRO_AUDIO_DEVICE*)value;
+   al_free(device->name);
+   al_free(device->identifier);
+   al_free(device);
+}
+
 static void sink_info_cb(pa_context *c, const pa_sink_info *i, int eol,
    void *userdata)
 {
    (void)c;
    (void)eol;
 
+   if (!output_device_list) {
+      output_device_list = _al_list_create();
+   }
+
    pa_sink_state_t *ret = userdata;
    if (!i)
       return;
    *ret = i->state;
+
+   int iden_len = strlen(i->name) + 1;
+   int name_len = strlen(i->description) + 1;
+   ALLEGRO_AUDIO_DEVICE* device = (ALLEGRO_AUDIO_DEVICE*)al_malloc(sizeof(ALLEGRO_AUDIO_DEVICE));
+   device->identifier = (void*)al_malloc(iden_len);
+   device->name = (char*)al_malloc(name_len);
+
+   strcpy(device->name, i->description);
+   strcpy(device->identifier, i->name);
+
+   _al_list_push_back_ex(output_device_list, device, _output_device_list_dtor);
 }
 
 static int pulseaudio_open(void)
@@ -144,6 +171,7 @@ static int pulseaudio_open(void)
       pa_mainloop_free(mainloop);
       return 1;
    }*/
+
    pa_operation_unref(op);
    pa_context_disconnect(c);
    pa_context_unref(c);
@@ -153,6 +181,7 @@ static int pulseaudio_open(void)
 
 static void pulseaudio_close(void)
 {
+   _al_list_destroy(output_device_list);
 }
 
 static void *pulseaudio_update(ALLEGRO_THREAD *self, void *data)
@@ -511,6 +540,11 @@ static void pulseaudio_deallocate_recorder(ALLEGRO_AUDIO_RECORDER *r)
    al_free(r->extra);
 }
 
+static _AL_LIST* pulseaudio_get_output_devices(void)
+{
+   return output_device_list;
+}
+
 ALLEGRO_AUDIO_DRIVER _al_kcm_pulseaudio_driver =
 {
    "PulseAudio",
@@ -533,7 +567,9 @@ ALLEGRO_AUDIO_DRIVER _al_kcm_pulseaudio_driver =
    pulseaudio_set_voice_position,
    
    pulseaudio_allocate_recorder,
-   pulseaudio_deallocate_recorder
+   pulseaudio_deallocate_recorder,
+
+   pulseaudio_get_output_devices
 };
 
 /* vim: set sts=3 sw=3 et: */
