@@ -19,21 +19,43 @@
 
 #define _AL_NO_BLEND_INLINE_FUNC
 
+#define ALLEGRO_INTERNAL_UNSTABLE
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_primitives.h"
+#include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_blend.h"
 #include "allegro5/internal/aintern_prim.h"
 #include "allegro5/internal/aintern_prim_soft.h"
 #include <math.h>
 
-static int fix_var(float var, int max_var)
+int _al_fix_texcoord(float var, int max_var, ALLEGRO_BITMAP_WRAP wrap)
 {
-   const int ivar = (int)floorf(var);
-   const int ret = ivar % max_var;
-   if(ret >= 0)
-      return ret;
-   else
-      return ret + max_var;
+   int ivar = (int)floorf(var);
+   float ret;
+   switch (wrap) {
+      case ALLEGRO_BITMAP_WRAP_CLAMP:
+         if (ivar < 0)
+            ret = 0;
+         else if (ivar > max_var - 1)
+            ret = max_var - 1;
+         else
+            ret = ivar;
+         break;
+      case ALLEGRO_BITMAP_WRAP_MIRROR: {
+         ret = ivar % max_var;
+         if(ret < 0)
+            ret += max_var;
+         if ((int)floorf((float)ivar / max_var) % 2)
+            ret = max_var - 1 - ret;
+         break;
+      }
+      default: {
+         ret = ivar % max_var;
+         if(ret < 0)
+            ret += max_var;
+      }
+   }
+   return ret;
 }
 
 void _al_point_2d(ALLEGRO_BITMAP* texture, ALLEGRO_VERTEX* v)
@@ -60,9 +82,11 @@ void _al_point_2d(ALLEGRO_BITMAP* texture, ALLEGRO_VERTEX* v)
    }
    
    if (texture) {
-      float U = fix_var(v->u, al_get_bitmap_width(texture));
-      float V = fix_var(v->v, al_get_bitmap_height(texture));
-      ALLEGRO_COLOR color = al_get_pixel(texture, U, V);
+      ALLEGRO_BITMAP_WRAP wrap_u, wrap_v;
+      _al_get_bitmap_wrap(texture, &wrap_u, &wrap_v);
+      int iu = _al_fix_texcoord(v->u, al_get_bitmap_width(texture), wrap_u);
+      int iv = _al_fix_texcoord(v->v, al_get_bitmap_height(texture), wrap_v);
+      ALLEGRO_COLOR color = al_get_pixel(texture, iu, iv);
 
       if(vc.r != 1 || vc.g != 1 || vc.b != 1 || vc.a != 1) {
          color.r *= vc.r;
