@@ -8,6 +8,8 @@ void _al_xwin_set_above(ALLEGRO_DISPLAY *display, int value);
 void _al_xwin_set_frame(ALLEGRO_DISPLAY *display, bool frame_on);
 void _al_xwin_set_icons(ALLEGRO_DISPLAY *d,
    int num_icons, ALLEGRO_BITMAP *bitmaps[]);
+void _al_xwin_maximize(ALLEGRO_DISPLAY *d, bool maximized);
+void _al_xwin_check_maximized(ALLEGRO_DISPLAY *display);
 
 #endif
 
