@@ -279,11 +279,6 @@ static bool _al_android_init_display(JNIEnv *env,
       return false;
    }
 
-   // XXX ret is never 2
-   if (ret == 2 && programmable_pipeline) {
-      d->flags &= ~ALLEGRO_PROGRAMMABLE_PIPELINE;
-   }
-
    ALLEGRO_DEBUG("calling egl_createSurface");
    if (!_jni_callBooleanMethodV(env, display->surface_object,
          "egl_createSurface", "()Z"))
@@ -491,6 +486,18 @@ static int select_best_visual_and_update(JNIEnv *env,
 
 /* driver implementation hooks */
 
+static bool android_set_display_flag(ALLEGRO_DISPLAY *dpy, int flag, bool onoff)
+{
+   (void)dpy; (void)flag; (void)onoff;
+
+   if (flag == ALLEGRO_FRAMELESS) {
+      _jni_callVoidMethodV(_al_android_get_jnienv(),
+         _al_android_activity_object(), "setAllegroFrameless", "(Z)V", onoff);
+   }
+   
+   return false;
+}
+
 static ALLEGRO_DISPLAY *android_create_display(int w, int h)
 {
    ALLEGRO_DEBUG("begin");
@@ -515,6 +522,14 @@ static ALLEGRO_DISPLAY *android_create_display(int w, int h)
    display->h = h;
    display->flags = flags;
 
+   display->flags |= ALLEGRO_OPENGL;
+#ifdef ALLEGRO_CFG_OPENGLES2
+   display->flags |= ALLEGRO_PROGRAMMABLE_PIPELINE;
+#endif
+#ifdef ALLEGRO_CFG_OPENGLES
+   display->flags |= ALLEGRO_OPENGL_ES_PROFILE;
+#endif
+
    _al_event_source_init(&display->es);
 
    /* Java thread needs this but it's thread local.
@@ -556,8 +571,6 @@ static ALLEGRO_DISPLAY *android_create_display(int w, int h)
       return NULL;
    }
 
-   display->flags |= ALLEGRO_OPENGL;
-
    ALLEGRO_DEBUG("display: %p %ix%i", display, display->w, display->h);
 
    _al_android_clear_current(_al_android_get_jnienv(), d);
@@ -567,6 +580,15 @@ static ALLEGRO_DISPLAY *android_create_display(int w, int h)
    android_set_display_option(display, ALLEGRO_SUPPORTED_ORIENTATIONS,
       al_get_new_display_option(ALLEGRO_SUPPORTED_ORIENTATIONS, NULL));
 
+   /* Fill in opengl version */
+   const int v = d->display.ogl_extras->ogl_info.version;
+   d->display.extra_settings.settings[ALLEGRO_OPENGL_MAJOR_VERSION] = (v >> 24) & 0xFF;
+   d->display.extra_settings.settings[ALLEGRO_OPENGL_MINOR_VERSION] = (v >> 16) & 0xFF;
+
+   if (flags & ALLEGRO_FRAMELESS) {
+      android_set_display_flag(display, ALLEGRO_FRAMELESS, true);
+   }
+
    ALLEGRO_DEBUG("end");
    return display;
 }
@@ -658,9 +680,16 @@ static void android_unset_current_display(ALLEGRO_DISPLAY *dpy)
 
 static void android_flip_display(ALLEGRO_DISPLAY *dpy)
 {
+   // Some Androids crash if you swap buffers with an fbo bound
+   // so temporarily change target to the backbuffer.
+   ALLEGRO_BITMAP *old_target = al_get_target_bitmap();
+   al_set_target_backbuffer(dpy);
+
    _jni_callVoidMethod(_al_android_get_jnienv(),
       ((ALLEGRO_DISPLAY_ANDROID *)dpy)->surface_object, "egl_SwapBuffers");
 
+   al_set_target_bitmap(old_target);
+
    /* Backup bitmaps created without ALLEGRO_NO_PRESERVE_TEXTURE that are
     * dirty, to system memory.
     */
@@ -753,12 +782,6 @@ static void android_get_window_position(ALLEGRO_DISPLAY *dpy, int *x, int *y)
    *x = *y = 0;
 }
 
-static bool android_set_display_flag(ALLEGRO_DISPLAY *dpy, int flag, bool onoff)
-{
-   (void)dpy; (void)flag; (void)onoff;
-   return false;
-}
-
 static bool android_wait_for_vsync(ALLEGRO_DISPLAY *dpy)
 {
    (void)dpy;
