@@ -79,22 +79,6 @@ static void timer_thread_proc(_AL_THREAD *self, void *unused)
    }
 #endif
 
-#ifdef ALLEGRO_QNX
-   /* thread priority adjustment for QNX:
-    *  The timer thread is set to the highest relative priority.
-    *  (see the comment in src/qnx/qsystem.c about the scheduling policy)
-    */
-   {
-      struct sched_param sparam;
-      int spolicy;
-
-      if (pthread_getschedparam(pthread_self(), &spolicy, &sparam) == EOK) {
-         sparam.sched_priority += 4;
-         pthread_setschedparam(pthread_self(), spolicy, &sparam);
-      }
-   }
-#endif
-
    double old_time = al_get_time();
    double new_time;
    double interval = 0.032768;
@@ -161,15 +145,17 @@ static void shutdown_timers(void)
 {
    ASSERT(_al_vector_size(&active_timers) == 0);
 
-   _al_vector_free(&active_timers);
-
    if (timer_thread != NULL) {
-      destroy_thread = true;
       al_lock_mutex(timers_mutex);
+      _al_vector_free(&active_timers);
+      destroy_thread = true;
       al_signal_cond(timer_cond);
       al_unlock_mutex(timers_mutex);
       _al_thread_join(timer_thread);
    }
+   else {
+      _al_vector_free(&active_timers);
+   }
 
    al_free(timer_thread);
 
