@@ -68,16 +68,20 @@ typedef struct WGL_DISPLAY_PARAMETERS {
 
 static bool is_wgl_extension_supported(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, const char *extension, HDC dc)
 {
-   int ret;
+   bool ret = false;
+   const GLubyte* extensions;
 
-   /* XXX deprecated in OpenGL 3.0 */
-   if (!glGetString(GL_EXTENSIONS))
-      return false;
-   if (!_wglGetExtensionsStringARB)
-      return false;
+   if (_wglGetExtensionsStringARB) {
+      extensions = (const GLubyte*)_wglGetExtensionsStringARB(dc);
+   }
+   else {
+      /* XXX deprecated in OpenGL 3.0 */
+      extensions = glGetString(GL_EXTENSIONS);
+   }
 
-   ret = _al_ogl_look_for_an_extension(extension,
-         (const GLubyte*)_wglGetExtensionsStringARB(dc));
+   if (extensions) {
+      ret = _al_ogl_look_for_an_extension(extension, extensions);
+   }
 
    return ret;
 }
@@ -621,7 +625,7 @@ static bool change_display_mode(ALLEGRO_DISPLAY *d)
 }
 
 
-static HGLRC init_ogl_context_ex(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, HDC dc, bool fc, int major, int minor)
+static HGLRC init_ogl_context_ex(HDC dc, bool fc, int major, int minor)
 {
    HWND testwnd = NULL;
    HDC testdc   = NULL;
@@ -642,6 +646,10 @@ static HGLRC init_ogl_context_ex(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExt
    if (!testrc)
       goto bail;
 
+   _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB
+      = (_ALLEGRO_wglGetExtensionsStringARB_t)wglGetProcAddress("wglGetExtensionsStringARB");
+   ALLEGRO_INFO("_wglGetExtensionsStringARB %p\n", _wglGetExtensionsStringARB);
+
    if (is_wgl_extension_supported(_wglGetExtensionsStringARB, "WGL_ARB_create_context", testdc)) {
       int attrib[] = {WGL_CONTEXT_MAJOR_VERSION_ARB, major,
                       WGL_CONTEXT_MINOR_VERSION_ARB, minor,
@@ -676,7 +684,7 @@ bail:
 }
 
 
-static ALLEGRO_EXTRA_DISPLAY_SETTINGS** get_available_pixel_formats_ext(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, int *count)
+static ALLEGRO_EXTRA_DISPLAY_SETTINGS** get_available_pixel_formats_ext(int *count)
 {
    HWND testwnd = NULL;
    HDC testdc   = NULL;
@@ -706,11 +714,15 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS** get_available_pixel_formats_ext(_ALLEGRO
    if (!testrc)
       goto bail;
 
+   _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB
+      = (_ALLEGRO_wglGetExtensionsStringARB_t)wglGetProcAddress("wglGetExtensionsStringARB");
+   ALLEGRO_INFO("_wglGetExtensionsStringARB %p\n", _wglGetExtensionsStringARB);
+
    if (!is_wgl_extension_supported(_wglGetExtensionsStringARB, "WGL_ARB_pixel_format", testdc) &&
        !is_wgl_extension_supported(_wglGetExtensionsStringARB, "WGL_EXT_pixel_format", testdc)) {
       ALLEGRO_ERROR("WGL_ARB/EXT_pf not supported.\n");
       goto bail;
-  }
+   }
 
    if (!init_pixel_format_extensions())
       goto bail;
@@ -815,7 +827,7 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS** get_available_pixel_formats_old(int *cou
 }
 
 
-static bool select_pixel_format(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, ALLEGRO_DISPLAY_WGL *d, HDC dc)
+static bool select_pixel_format(ALLEGRO_DISPLAY_WGL *d, HDC dc)
 {
    ALLEGRO_EXTRA_DISPLAY_SETTINGS **eds = NULL;
    ALLEGRO_CONFIG *sys_cfg = al_get_system_config();
@@ -836,7 +848,7 @@ static bool select_pixel_format(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExte
    }
 
    if (!force_old)
-      eds = get_available_pixel_formats_ext(_wglGetExtensionsStringARB, &eds_count);
+      eds = get_available_pixel_formats_ext(&eds_count);
    if (!eds)
       eds = get_available_pixel_formats_old(&eds_count, dc);
 
@@ -877,7 +889,7 @@ static bool select_pixel_format(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExte
    return true;
 }
 
-static bool create_display_internals(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, ALLEGRO_DISPLAY_WGL *wgl_disp)
+static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
 {
    ALLEGRO_DISPLAY     *disp     = (void*)wgl_disp;
    ALLEGRO_DISPLAY_WIN *win_disp = (void*)wgl_disp;
@@ -919,7 +931,7 @@ static bool create_display_internals(_ALLEGRO_wglGetExtensionsStringARB_t _wglGe
    /* WGL display lists cannot be shared with the API currently in use. */
    disp->ogl_extras->is_shared = false;
 
-   if (!select_pixel_format(_wglGetExtensionsStringARB, wgl_disp, wgl_disp->dc)) {
+   if (!select_pixel_format(wgl_disp, wgl_disp->dc)) {
       destroy_display_internals(wgl_disp);
       return false;
    }
@@ -932,7 +944,7 @@ static bool create_display_internals(_ALLEGRO_wglGetExtensionsStringARB_t _wglGe
       if (major == 0)
          major = 3;
       bool fc = (disp->flags & ALLEGRO_OPENGL_FORWARD_COMPATIBLE) != 0;
-      wgl_disp->glrc = init_ogl_context_ex(_wglGetExtensionsStringARB, wgl_disp->dc, fc, major,
+      wgl_disp->glrc = init_ogl_context_ex(wgl_disp->dc, fc, major,
          minor);
    }
    else {
@@ -968,7 +980,7 @@ static bool create_display_internals(_ALLEGRO_wglGetExtensionsStringARB_t _wglGe
    /* Fill in the display settings for opengl major and minor versions...*/
    const int v = disp->ogl_extras->ogl_info.version;
    disp->extra_settings.settings[ALLEGRO_OPENGL_MAJOR_VERSION] = (v >> 24) & 0xFF;
-   disp->extra_settings.settings[ALLEGRO_OPENGL_MINOR_VERSION] = (v >> 16) & 0xFF;   
+   disp->extra_settings.settings[ALLEGRO_OPENGL_MINOR_VERSION] = (v >> 16) & 0xFF;
 
    /* Try to enable or disable vsync as requested */
    /* NOTE: my drivers claim I don't have WGL_EXT_swap_control
@@ -1022,9 +1034,7 @@ static ALLEGRO_DISPLAY* wgl_create_display(int w, int h)
 
    display->ogl_extras = al_calloc(1, sizeof(ALLEGRO_OGL_EXTRAS));
 
-   _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB
-      = (_ALLEGRO_wglGetExtensionsStringARB_t)wglGetProcAddress("wglGetExtensionsStringARB");
-   if (!create_display_internals(_wglGetExtensionsStringARB, wgl_display)) {
+   if (!create_display_internals(wgl_display)) {
       al_free(display->ogl_extras);
       al_free(display);
       return NULL;
@@ -1158,7 +1168,6 @@ static void wgl_unset_current_display(ALLEGRO_DISPLAY *d)
    }
 }
 
-
 /*
  * The window must be created in the same thread that
  * runs the message loop.
@@ -1228,53 +1237,57 @@ static void display_thread_proc(void *arg)
    }
 
    if (disp->flags & ALLEGRO_FULLSCREEN_WINDOW) {
-      bool frameless = true;
-      _al_win_set_window_frameless(disp, win_disp->window, frameless);
+      _al_win_set_window_frameless(disp, win_disp->window, true);
    }
 
-   /* Yep, the following is really needed sometimes. */
-   /* ... Or is it now that we have dumped DInput? */
-   /* <rohannessian> Win98/2k/XP's window forground rules don't let us
-    * make our window the topmost window on launch. This causes issues on
-    * full-screen apps, as DInput loses input focus on them.
-    * We use this trick to force the window to be topmost, when switching
-    * to full-screen only. Note that this only works for Win98 and greater.
-    * Win95 will ignore our SystemParametersInfo() calls.
-    *
-    * See http://support.microsoft.com:80/support/kb/articles/Q97/9/25.asp
-    * for details.
-    */
    {
       DWORD lock_time;
       HWND wnd = win_disp->window;
 
+      if ((disp->flags & ALLEGRO_FULLSCREEN) == 0) {
+         ShowWindow(wnd, SW_SHOWNORMAL);
+         SetForegroundWindow(wnd);
+         UpdateWindow(wnd);
+      }
+      else {
+
+         /* Yep, the following is really needed sometimes. */
+         /* ... Or is it now that we have dumped DInput? */
+         /* <rohannessian> Win98/2k/XP's window foreground rules don't let us
+          * make our window the topmost window on launch. This causes issues on
+          * full-screen apps, as DInput loses input focus on them.
+          * We use this trick to force the window to be topmost, when switching
+          * to full-screen only. Note that this only works for Win98 and greater.
+          * Win95 will ignore our SystemParametersInfo() calls.
+          *
+          * See http://support.microsoft.com:80/support/kb/articles/Q97/9/25.asp
+          * for details.
+          */
+
 #define SPI_GETFOREGROUNDLOCKTIMEOUT 0x2000
 #define SPI_SETFOREGROUNDLOCKTIMEOUT 0x2001
-      if (disp->flags & ALLEGRO_FULLSCREEN) {
          SystemParametersInfo(SPI_GETFOREGROUNDLOCKTIMEOUT,
                0, (LPVOID)&lock_time, 0);
          SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT,
                0, (LPVOID)0, SPIF_SENDWININICHANGE | SPIF_UPDATEINIFILE);
-      }
 
-      ShowWindow(wnd, SW_SHOWNORMAL);
-      SetForegroundWindow(wnd);
-      /* In some rare cases, it doesn't seem to work without the loop. And we
-       * absolutely need this to succeed, else we trap the user in a
-       * fullscreen window without input.
-       */
-      while (GetForegroundWindow() != wnd) {
-         al_rest(0.01);
+         ShowWindow(wnd, SW_SHOWNORMAL);
          SetForegroundWindow(wnd);
-      }
-      UpdateWindow(wnd);
+         /* In some rare cases, it doesn't seem to work without the loop. And we
+          * absolutely need this to succeed, else we trap the user in a
+          * fullscreen window without input.
+          */
+         while (GetForegroundWindow() != wnd) {
+            al_rest(0.1);
+            SetForegroundWindow(wnd);
+         }
+         UpdateWindow(wnd);
 
-      if (disp->flags & ALLEGRO_FULLSCREEN) {
          SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT,
               0, (LPVOID)&lock_time, SPIF_SENDWININICHANGE | SPIF_UPDATEINIFILE);
-      }
 #undef SPI_GETFOREGROUNDLOCKTIMEOUT
 #undef SPI_SETFOREGROUNDLOCKTIMEOUT
+      }
    }
 
 #if 0
@@ -1338,7 +1351,7 @@ static void wgl_update_display_region(ALLEGRO_DISPLAY *d,
 }
 
 
-static bool wgl_resize_helper(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB, ALLEGRO_DISPLAY *d, int width, int height)
+static bool wgl_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
 {
    ALLEGRO_DISPLAY_WGL *wgl_disp = (ALLEGRO_DISPLAY_WGL *)d;
    ALLEGRO_DISPLAY_WIN *win_disp = (ALLEGRO_DISPLAY_WIN *)d;
@@ -1387,7 +1400,7 @@ static bool wgl_resize_helper(_ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtens
 
       d->w = width;
       d->h = height;
-      if(!create_display_internals(_wglGetExtensionsStringARB, wgl_disp))
+      if(!create_display_internals(wgl_disp))
          return false;
 
       /* We have a new backbuffer now. */
@@ -1442,10 +1455,8 @@ static bool wgl_resize_display(ALLEGRO_DISPLAY *d, int width, int height)
 
    win_display->ignore_resize = true;
 
-   _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB
-      = (_ALLEGRO_wglGetExtensionsStringARB_t)wglGetProcAddress("wglGetExtensionsStringARB");
-   if (!wgl_resize_helper(_wglGetExtensionsStringARB, d, width, height)) {
-      wgl_resize_helper(_wglGetExtensionsStringARB, d, orig_w, orig_h);
+   if (!wgl_resize_helper(d, width, height)) {
+      wgl_resize_helper(d, orig_w, orig_h);
       ret = false;
    } else {
       ret = true;
