@@ -57,6 +57,9 @@ static bool using_higher_res_timer;
 
 static ALLEGRO_SYSTEM_WIN *_al_win_system;
 
+/* This is mostly here for _al_display_d3d_driver, which is stateful. */
+static ALLEGRO_MUTEX* win_mutex;
+
 /* _WinMain:
  *  Entry point for Windows GUI programs, hooked by a macro in alwin.h,
  *  which makes it look as if the application can still have a normal
@@ -117,7 +120,7 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
          while ((argbuf[i]) && ((q) ? (argbuf[i] != q) : (!isspace(argbuf[i]))))
             i++;
 
-            if (argbuf[i]) {
+         if (argbuf[i]) {
             argbuf[i] = 0;
             i++;
          }
@@ -202,6 +205,8 @@ static ALLEGRO_SYSTEM *win_initialize(int flags)
 
    _al_win_system->system.vt = vt;
 
+   win_mutex = al_create_mutex();
+
    return &_al_win_system->system;
 }
 
@@ -234,6 +239,8 @@ static void win_shutdown(void)
    al_free(vt);
    vt = NULL;
 
+   al_destroy_mutex(win_mutex);
+
    ASSERT(_al_win_system);
    al_free(_al_win_system);
 }
@@ -269,7 +276,9 @@ static ALLEGRO_DISPLAY_INTERFACE *win_get_display_driver(void)
    /* Programmatic selection. */
 #ifdef ALLEGRO_CFG_D3D
    if (flags & ALLEGRO_DIRECT3D_INTERNAL) {
+      al_lock_mutex(win_mutex);
       ALLEGRO_DISPLAY_INTERFACE* iface = _al_display_d3d_driver();
+      al_unlock_mutex(win_mutex);
       if (iface == NULL)
          ALLEGRO_WARN("Direct3D graphics driver not available.\n");
       return iface;
@@ -290,7 +299,9 @@ static ALLEGRO_DISPLAY_INTERFACE *win_get_display_driver(void)
       ALLEGRO_DEBUG("Configuration value graphics.driver = %s\n", s);
       if (0 == _al_stricmp(s, "DIRECT3D") || 0 == _al_stricmp(s, "D3D")) {
 #ifdef ALLEGRO_CFG_D3D
+         al_lock_mutex(win_mutex);
          ALLEGRO_DISPLAY_INTERFACE* iface = _al_display_d3d_driver();
+         al_unlock_mutex(win_mutex);
          if (iface != NULL) {
             al_set_new_display_flags(flags | ALLEGRO_DIRECT3D_INTERNAL);
             return iface;
@@ -312,7 +323,9 @@ static ALLEGRO_DISPLAY_INTERFACE *win_get_display_driver(void)
    /* XXX is implicitly setting new_display_flags the desired behaviour? */
 #ifdef ALLEGRO_CFG_D3D
    {
+      al_lock_mutex(win_mutex);
       ALLEGRO_DISPLAY_INTERFACE* iface = _al_display_d3d_driver();
+      al_unlock_mutex(win_mutex);
       if (iface != NULL) {
          al_set_new_display_flags(flags | ALLEGRO_DIRECT3D_INTERNAL);
          return iface;
@@ -545,9 +558,11 @@ static bool win_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO *info)
    return true;
 }
 
-BOOL CALLBACK monitor_enum_proc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
+static BOOL CALLBACK monitor_enum_proc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
 {
    HMONITOR *h_monitor = (HMONITOR *) dwData;
+   (void)hdcMonitor;
+   (void)lprcMonitor;
 
    if (hMonitor) {
       *h_monitor = hMonitor;
@@ -576,8 +591,8 @@ static int win_get_monitor_dpi(int adapter)
 {
    ALLEGRO_MONITOR_INFO info;
    HMODULE shcore_dll = _al_win_safe_load_library("shcore.dll");
-   int dpi_hori;
-   int dpi_vert;
+   UINT dpi_hori;
+   UINT dpi_vert;
 
    typedef enum _AL_MONITOR_DPI_TYPE {
        AL_MDT_EFFECTIVE_DPI  = 0,
