@@ -1,9 +1,9 @@
 # bmp suite 1
-wget -N http://entropymine.com/jason/bmpsuite/bmpsuite.zip
+wget -N https://download.tuxfamily.org/allegro/files/bmpsuite/bmpsuite.zip
 unzip -u -d bmpsuite bmpsuite.zip
 
 # bmp suite 2
-wget --no-check-certificate -N http://downloads.sourceforge.net/project/bmptestsuite/bmptestsuite/bmptestsuite-0.9/bmptestsuite-0.9.zip
+wget -N https://download.tuxfamily.org/allegro/files/bmpsuite/bmptestsuite-0.9.zip
 unzip -u bmptestsuite-0.9.zip
 
 # bmp suite 3
@@ -12,6 +12,6 @@ wget -N -P wvnet https://download.tuxfamily.org/allegro/files/bmpsuite/wvnet/tes
 wget -N -P wvnet https://download.tuxfamily.org/allegro/files/bmpsuite/wvnet/trans.bmp
 
 # bmp suite 4
-wget -N http://entropymine.com/jason/bmpsuite/releases/bmpsuite-2.4.zip
+wget -N https://download.tuxfamily.org/allegro/files/bmpsuite/bmpsuite-2.4.zip
 unzip -u bmpsuite-2.4.zip
 mv bmpsuite-2.4 bmpsuite2
